--------------------------------------------------------------------------------
-- CLOPWH : Combacsa's Lua ObjectProtocol implementation With Hongminhee. <br />
-- Author : combacsa (combacsa@gmail.com, aka Byun, Kyuhong aka Francisco)<br />
-- Supervised by dahlia (minhee@dahlia.kr, aka Hong, Minhee) <br /><br />
----------------------------------------------------------------------------------
-- <h1>CLOPWH Class Hierarchy</h1>
-- <p>
-- <ul>
-- <li> Object
--   <ul>
--   <li>Number</li>
--   <li>String</li>
--   <li>Boolean</li>
--   <li>Nil</li>
--   <li>Function</li>
--   <li>Thread</li>
--   <li>Userdata</li>
--   <li>Table</li>
--   <li>Class</li>
--   <li>[User-defined Classes]</li>
--   </ul>
-- </li>
-- </ul>
-- </p>
-- <h1>Metaclass Concept (In addition to Class Hierarchy)</h1>
-- <p>
-- Type of Object, Class, and all the other Classes including user-defined
-- ones, is Class. Those are all instance of Class.
-- </p>
-- <h1>sample code</h1>
-- <p>
-- <code>
-- classname = clopwh.Class({
--	initialize=function()
--	end,
--
--	function1_name=function(params)
--		--implementation
--	end,
--	...
-- })
-- </code>
-- </p>
-- <p>See <code>src/test/hong.lua</code> or <code>src/test/test.lua</code> also.</p>
---------------------------------------------------------------------------

local core = require "clopwh.core"

module("clopwh")

_COPYRIGHT = "Copyright (c) Software Maestro Program"
_DESCRIPTION = "Object Protocol Implementation for the Lua language"
_VERSION = "CLOPWH 0.1"

---------------------------------------------------------------------------
-- Access Control
---------------------------------------------------------------------------

Object = core.Object

--------------------------------------------------------------------------------
-- Define a new Class and Return a constructor for defined class.
-- @class function
-- @name Class
-- @param declaration Class declaration, usually consists of function definition.
-- <br />Declaration might include several special functions.
-- <ul>
-- <li>initialize: Constructor</li>
-- <li>*, -, /, %, ^, <=, <, ==, ..: Binary operator overloading</li>
-- <li>-, tostring: Unary operator overloading</li>
-- </ul>
-- All function definition must obey python-like rule: First parameter must be
-- <code>self</code>, and it refers to the caller object.
-- @return a function which is a constructor for given declaration.
Class = core.Class

Number = core.Number
String = core.String
Boolean = core.Boolean
Table = core.Table
Nil = core.Nil
Function = core.Function
Thread = core.Thread
Userdata = core.Userdata

--------------------------------------------------------------------------------
-- Returns whether given object belongs to the given class or not.
-- @class function
-- @name is_a
-- @param object     Object to test whether it belongs to given Class
-- @param class_name Class (function) to test whether given object belongs to it
-- @return whether given objec is of given class (according to inheritance)
-- @see clopwh.core.is_a
is_a = core.is_a

--------------------------------------------------------------------------------
-- Returns the type (Class) of given object.
-- @class function
-- @name get_type
-- @param object Object to know the class
-- @return Class for the object
-- @see clopwh.core.get_type
get_type = core.get_type
