-- Hong, Minhee's test cases

local clopwh = require 'clopwh'

-- Part 1. Class Declaration

xyz = clopwh.Class({
	initialize = function(self)
		self.abc = 123
	end,
	getabc = function(self)
		return self.abc
	end
})

a = xyz()
print(a.getabc()) --> 123

-- Part 2. Class Inheritance

child = clopwh.Class {
	getabc = function(self)
		return self.abc + 1
	end,
	inherits = {xyz}
}

c = child()
print(c.getabc()) --> 124

-- Part 3. Multiple Inheritance

child2 = clopwh.Class({
	getabc = function(self)
		return self.abc * 2
	end,
	initialize = function(self, value)
		self.abc = value
	end,
	inherits = {xyz, child}
})

c2 = child2(123)
print(c2.getabc()) --> 246

-- Part 4. is-a relationship

print(clopwh.is_a(a, xyz)) --> true
print(clopwh.is_a(c, xyz)) --> true
print(clopwh.is_a(c2, xyz)) --> true

print(clopwh.is_a(a, child)) --> false
print(clopwh.is_a(c, child)) --> true
print(clopwh.is_a(c2, child)) --> true

print(clopwh.is_a(a, child2)) --> false
print(clopwh.is_a(c, child2)) --> false
print(clopwh.is_a(c2, child2)) --> true

-- Part 5. Primitive type adaptor

print(clopwh.is_a({}, clopwh.Table)) --> true
print(clopwh.is_a(1, clopwh.Number)) --> true
print(clopwh.is_a(1, clopwh.String)) --> false
print(clopwh.is_a(1, clopwh.Table))  --> false
print(clopwh.is_a(c, clopwh.Table))  --> false
print(clopwh.is_a(c, clopwh.Number)) --> false

-- Part 6. type reflection

print(clopwh.get_type({}) == clopwh.Table)
print(clopwh.get_type(1) == clopwh.Number)
print(clopwh.get_type("s") == clopwh.String)
print(clopwh.get_type(a) == xyz)
print(clopwh.get_type(c) == child)
print(clopwh.get_type(c2) == child2)
print(clopwh.get_type(nil) == clopwh.Nil)
print(clopwh.get_type(true) == clopwh.Boolean)

--[[

Part 3.5. C3 Linealization of Method Resolution Ordering

http://www.python.org/download/releases/2.3/mro/
( http://192.220.96.201/dylan/linearization-oopsla96.html )

Implemented.

TODO: 적절한 예를 구현할 것

Part 3.6. Default Initializer for Multiple class inheritance

TODO: 어떻게 해야 하는지 홍멘토님의 설명을 듣고 구현할 것

Part 3.7. Calling superclass' constructor

TODO: 설계하고 그대로 만들어 보자

Part 7. Metaclass
--]]

print(clopwh.is_a(child, clopwh.Class))        --> true
print(clopwh.is_a(clopwh.Table, clopwh.Class)) --> true
print(clopwh.is_a(clopwh.Class, clopwh.Class)) --> true

print(clopwh.get_type(xyz) == clopwh.Class) --> true
print(clopwh.get_type(clopwh.Number) == clopwh.Class) --> true
print(clopwh.get_type(clopwh.Table)  == clopwh.Class) --> true
print(clopwh.get_type(clopwh.Class)  == clopwh.Class) --> true

print(clopwh.is_a(clopwh.Class, clopwh.Object)) --> true
print(clopwh.is_a(clopwh.Object, clopwh.Class)) --> true

-- Part 8. Operator Overloading

mytype = clopwh.Class {
  initialize = function(self, a)
    self.a = a
  end,
  ["+"] = function(self, operand)
    return mytype(self.a + operand.a)
  end,
  tostring = function(self)
    return tostring(self.a)
  end,
  ["=="] = function(self, operand)
    return self.a == operand.a
  end,
}

m = mytype(1)
m2 = mytype(2)
print(m + m2)  --> 3
print(m == m2) --> false
m3 = mytype(3)
print((m + m2) == m3) --> true

oo1 = xyz()
oo2 = xyz()
oo3 = oo1

print(oo1 == oo3)
print(oo1 == oo2)

--[[
Part 9* private, public 한정자

TODO: 몰라.

Part 10* static method

TODO: 몰라.

--]]


-- MRO Validation
-- http://192.220.96.201/dylan/figure5-panes.gif
pane = clopwh.Class({
	name=function(self)
		return "pane"
	end,
	name2=function(self)
		return "pane"
	end,
	name3=function(self)
		return "pane"
	end,
	name4=function(self)
		return "pane"
	end,
})

editing_mixin = clopwh.Class({
	name=function(self)
		return "editing_mixin"
	end,
	name2=function(self)
		return "editing_mixin"
	end,
	name3=function(self)
		return "editing_mixin"
	end,
	name4=function(self)
		return "editing_mixin"
	end,
	name5=function(self)
		return "editing_mixin"
	end,
	name6=function(self)
		return "editing_mixin"
	end,
})

scrolling_mixin = clopwh.Class({
	name=function(self)
		return "scrolling_mixin"
	end,
	name2=function(self)
		return "scrolling_mixin"
	end,
	name3=function(self)
		return "scrolling_mixin"
	end,
	name4=function(self)
		return "scrolling_mixin"
	end,
	name5=function(self)
		return "scrolling_mixin"
	end,
})

scrollable_pane = clopwh.Class({
	inherits={pane, scrolling_mixin},
	name=function(self)
		return "scrollable_pane"
	end,
	name2=function(self)
		return "scrollable_pane"
	end,
})

editable_pane = clopwh.Class({
	inherits={pane, editing_mixin},
	name=function(self)
		return "editable_pane"
	end,
	name2=function(self)
		return "editable_pane"
	end,
	name3=function(self)
		return "editable_pane"
	end,
})

editable_scrollable_pane = clopwh.Class({
	inherits={scrollable_pane, editable_pane},
	name=function(self)
		return "editable_scrollable_pane"
	end,
})

esp = editable_scrollable_pane()
print(esp.name())  -- esp
print(esp.name2()) -- sp
print(esp.name3()) -- ep
print(esp.name4()) -- p
print(esp.name5()) -- s
print(esp.name6()) -- e

-- Checking primitive type adaptor

a = clopwh.Number("123")
print(a == 123)
a = clopwh.String(123)
print(a == "123")
a = clopwh.Table(123)
print(a == nil)
a = clopwh.Nil(123)
print(a == nil)
a = clopwh.Table({a=123})
print(a.a == 123)
